@@ -1,34 +1,10 @@
-.onLoad <- function(lib, pkg)
-{
-    if ("package:nlme" %in% search()) {
-        stop(paste("Package lme4 conflicts with package nlme.\n",
-                   "To attach lme4 you must restart R without package nlme."))
-    }
-}
+## Class definitions for the package
 
 setOldClass("data.frame")
 setOldClass("family")
 setOldClass("logLik")
 setOldClass("terms")
 
-setClass("lme", representation(call = "call",
-                               facs = "list",
-                               x = "list",
-                               model = "data.frame",
-                               REML = "logical",
-                               rep = "ssclme",
-                               fitted = "numeric",
-                               residuals = "numeric",
-                               terms = "terms",
-                               assign = "integer"))
-
-setClass("GLMM", representation(family = "family",
-                                logLik = "numeric",
-                                fixef = "numeric",
-                                Hessian = "matrix",
-                                method = "character"),
-         contains = "lme")
-
 setClass("lmList",
          representation(call = "call",
                         pool = "logical"),
@@ -66,36 +42,27 @@ setClass("lmer",
                         status = "logical",
                         call = "call",
                         terms = "terms",
-                        assign = "integer"
+                        assign = "integer",
+                        fitted = "numeric",
+                        residuals = "numeric"
                         ),
          validity = function(object) {
              .Call("lmer_validate", object, PACKAGE = "Matrix")
          })
 
-setClass("summary.ssclme",
-         representation(coefficients="matrix",
-                        scale="numeric",
-                        denomDF="integer",
-                        REML="logical",
-                        ngrps="integer",
-                        nobs="integer",
-                        corFixed="corrmatrix",
-                        VarCorr="VarCorr",
-                        useScale="logical",
-                        showCorrelation="logical"
-                        ))
-
-setClass("summary.lme",
-         representation(call = "call",
-                        logLik = "logLik",
-                        re = "summary.ssclme",
-                        residuals = "numeric"))
-
-setClass("summary.GLMM", representation(family = "family"),
-         contains = "summary.lme")
-
 setClass("summary.lmer",
          representation(useScale="logical",
                         showCorrelation="logical"),
          contains = "lmer")
 
+setClass("lmList.confint", contains = "array")
+
+setClass("lmer.ranef", representation(varFac = "list", stdErr =
+                                      "numeric"),
+         contains = "list")
+
+setClass("lmer.ranef.confint", contains = "list")
+
+setClass("lmer.coef", representation(varFac = "list", stdErr =
+                                      "numeric"),
+         contains = "list")
