@@ -1,6 +1,33 @@
+## Methods for lmer and for the objects that it produces
+
+## Some utilities
+
 contr.SAS <- function(n, contrasts = TRUE)
-{
-    contr.treatment(n, if (is.numeric(n) && length(n) == 1) n else length(n), contrasts)
+## Eliminate this function after R-2.1.0 is released
+    contr.treatment(n,
+                    if (is.numeric(n) && length(n) == 1) n else length(n),
+                    contrasts)
+
+Lind <- function(i,j) {
+    if (i < j) stop(paste("Index i=", i,"must be >= index j=", j))
+    ((i - 1) * i)/2 + j
+}
+
+Dhalf <- function(from) {
+    D <- from@D
+    nf <- length(D)
+    Gp <- from@Gp
+    res <- array(0, rep(Gp[nf+1],2))
+    for (i in 1:nf) {
+        DD <- D[[i]]
+        dd <- dim(DD)
+        for (k in 1:dd[3]) {
+            mm <- array(DD[ , , k], dd[1:2])
+            base <- Gp[i] + (k - 1)*dd[1]
+            res[cbind(c(base + row(mm)), c(base + col(mm)))] <- c(mm)
+        }
+    }
+    res
 }
 
 lmerControl <-                            # Control parameters for lmer
@@ -11,7 +38,6 @@ lmerControl <-                            # Control parameters for lmer
            msTol = sqrt(.Machine$double.eps),
            msVerbose = getOption("verbose"),
            PQLmaxIt = 20,
-           .relStep = (.Machine$double.eps)^(1/3),
            EMverbose = getOption("verbose"),
            analyticGradient = TRUE,
            analyticHessian=FALSE)
@@ -23,15 +49,14 @@ lmerControl <-                            # Control parameters for lmer
          msTol = msTol,
          msVerbose = msVerbose,
          PQLmaxIt = PQLmaxIt,
-         .relStep = .relStep,
          EMverbose=EMverbose,
          analyticHessian=analyticHessian,
          analyticGradient=analyticGradient)
 }
 
-setMethod("lmer", signature(formula = "formula"),
-          function(formula, data,
-                   method = c("REML", "ML"),
+setMethod("lmer", signature(formula = "formula", family = "missing"),
+          function(formula, data, family,
+                   method = c("REML", "ML", "PQL", "Laplace", "AGQ"),
                    control = list(),
                    subset, weights, na.action, offset,
                    model = TRUE, x = FALSE, y = FALSE, ...)
@@ -76,11 +101,12 @@ setMethod("lmer", signature(formula = "formula"),
                      .fixed = list(cbind(Xmat, .response = model.response(frm))))
           ## FIXME: Use Xfrm and Xmat to get the terms and assign
           ## slots, pass them to lmer_create, then destroy them
-          obj <- .Call("lmer_create", lapply(random, "[[", 2), mmats, PACKAGE = "Matrix")
-          obj@terms <- attr(model.frame(fixed.form, data), "terms")
-          obj@assign <- attr(Xmat, "assign")
-          obj@call <- match.call()
-          obj@REML <- REML
+          obj <- .Call("lmer_create", lapply(random, "[[", 2),
+                       mmats, PACKAGE = "Matrix")
+          slot(obj, "terms") <- attr(model.frame(fixed.form, data), "terms")
+          slot(obj, "assign") <- attr(Xmat, "assign")
+          slot(obj, "call") <- match.call()
+          slot(obj, "REML") <- REML
           rm(Xmat)
           .Call("lmer_initial", obj, PACKAGE="Matrix")
           .Call("lmer_ECMEsteps", obj, 
@@ -89,10 +115,10 @@ setMethod("lmer", signature(formula = "formula"),
                 controlvals$EMverbose,
                 PACKAGE = "Matrix")
           LMEoptimize(obj) <- controlvals
-          #fitted <- .Call("ssclme_fitted", obj, facs, mmats, TRUE, PACKAGE = "Matrix")
-          #residuals <- mmats$.Xy[,".response"] - fitted
-          #if (as.logical(x)[1]) x <- mmats else x = list()
-          #rm(mmats)
+          slot(obj, "residuals") <-
+              unname(model.response(frm) -
+                     (slot(obj, "fitted") <-
+                      .Call("lmer_fitted", obj, mmats, TRUE, PACKAGE = "Matrix")))
           obj
       })
 
@@ -128,8 +154,19 @@ setReplaceMethod("LMEoptimize", signature(x="lmer", value="list"),
              })
 
 setMethod("ranef", signature(object = "lmer"),
-          function(object, ...) {
-              .Call("lmer_ranef", object, PACKAGE = "Matrix")
+          function(object, accumulate = FALSE, ...) {
+              val <- new("lmer.ranef",
+                         lapply(.Call("lmer_ranef", object, PACKAGE = "Matrix"),
+                                data.frame, check.names = FALSE),
+                         varFac = object@bVar,
+                         stdErr = .Call("lmer_sigma", object,
+                         object@REML, PACKAGE = "Matrix"))
+              if (!accumulate || length(val@varFac) == 1) return(val)
+              ## check for nested factors
+              L <- object@L
+              if (any(sapply(seq(a = val), function(i) length(L[[Lind(i,i)]]@i))))
+                  error("Require nested grouping factors to accumulate random effects")
+              val
           })
 
 setMethod("fixef", signature(object = "lmer"),
@@ -245,6 +282,129 @@ setMethod("show", "summary.lmer",
               invisible(object)
           })
 
+setMethod("lmer", signature(formula = "formula"),
+          function(formula, family, data,
+                   method = c("REML", "ML", "PQL", "Laplace", "AGQ"),
+                   control = list(),
+                   subset, weights, na.action, offset,
+                   model = TRUE, x = FALSE, y = FALSE, ...)
+      {
+          gVerb <- getOption("verbose")
+                                        # match and check parameters
+          controlvals <- do.call("lmerControl", control)
+          controlvals$REML <- FALSE
+          if (length(formula) < 3) stop("formula must be a two-sided formula")
+
+          ## initial glm fit
+          mf <- match.call()            
+          m <- match(c("family", "data", "subset", "weights",
+                       "na.action", "offset"),
+                     names(mf), 0)
+          mf <- mf[c(1, m)]
+          mf[[1]] <- as.name("glm")
+          fixed.form <- nobars(formula)
+          if (!inherits(fixed.form, "formula")) fixed.form <- ~ 1 # default formula
+          environment(fixed.form) <- environment(formula)
+          mf$formula <- fixed.form
+          mf$x <- mf$model <- mf$y <- TRUE
+          glm.fit <- eval(mf, parent.frame())
+          family <- glm.fit$family
+          ## Note: offset is on the linear scale
+          offset <- glm.fit$offset
+          if (is.null(offset)) offset <- 0
+          weights <- sqrt(abs(glm.fit$prior.weights))
+          ## initial 'fitted' values on linear scale
+          etaold <- eta <- glm.fit$linear.predictors
+          
+          ## evaluation of model frame
+          mf$x <- mf$model <- mf$y <- mf$family <- NULL
+          mf$drop.unused.levels <- TRUE
+          this.form <- subbars(formula)
+          environment(this.form) <- environment(formula)
+          mf$formula <- this.form
+          mf[[1]] <- as.name("model.frame")
+          frm <- eval(mf, parent.frame())
+          
+          ## grouping factors and model matrices for random effects
+          bars <- findbars(formula[[3]])
+          random <-
+              lapply(bars,
+                     function(x) list(model.matrix(eval(substitute(~term,
+                                                                   list(term=x[[2]]))),
+                                                   frm),
+                                      eval(substitute(as.factor(fac)[,drop = TRUE],
+                                                      list(fac = x[[3]])), frm)))
+          names(random) <- unlist(lapply(bars, function(x) deparse(x[[3]])))
+          
+          ## order factor list by decreasing number of levels
+          nlev <- sapply(random, function(x) length(levels(x[[2]])))
+          if (any(diff(nlev) > 0)) {
+              random <- random[rev(order(nlev))]
+          }
+          mmats <- c(lapply(random, "[[", 1),
+                     .fixed = list(cbind(glm.fit$x, .response = glm.fit$y)))
+          ## FIXME: Use Xfrm and Xmat to get the terms and assign
+          ## slots, pass these to lmer_create, then destroy Xfrm, Xmat, etc.
+          obj <- .Call("lmer_create", lapply(random, "[[", 2), mmats, PACKAGE = "Matrix")
+          obj@terms <- attr(glm.fit$model, "terms")
+          obj@assign <- attr(glm.fit$x, "assign")
+          obj@call <- match.call()
+          obj@REML <- FALSE
+          rm(glm.fit)
+          .Call("lmer_initial", obj, PACKAGE="Matrix")
+          mmats.unadjusted <- mmats
+          mmats[[1]][1,1] <- mmats[[1]][1,1]
+          conv <- FALSE
+          firstIter <- TRUE
+          msMaxIter.orig <- controlvals$msMaxIter
+          responseIndex <- ncol(mmats$.fixed)
+
+          for (iter in seq(length = controlvals$PQLmaxIt))
+          {
+              mu <- family$linkinv(eta)
+              dmu.deta <- family$mu.eta(eta)
+              ## weights (note: weights is already square-rooted)
+              w <- weights * dmu.deta / sqrt(family$variance(mu))
+              ## adjusted response (should be comparable to X \beta, not including offset
+              z <- eta - offset + (mmats.unadjusted$.fixed[, responseIndex] - mu) / dmu.deta
+              .Call("nlme_weight_matrix_list",
+                    mmats.unadjusted, w, z, mmats, PACKAGE="Matrix")
+              .Call("lmer_update_mm", obj, mmats, PACKAGE="Matrix")
+              if (firstIter) {
+                  .Call("lmer_initial", obj, PACKAGE="Matrix")
+                  if (gVerb) cat(" PQL iterations convergence criterion\n")
+              }
+              .Call("lmer_ECMEsteps", obj, 
+                    controlvals$niterEM,
+                    FALSE,
+                    controlvals$EMverbose,
+                    PACKAGE = "Matrix")
+              LMEoptimize(obj) <- controlvals
+              eta[] <- offset + ## FIXME: should the offset be here ?
+                  .Call("lmer_fitted", obj,
+                        mmats.unadjusted, TRUE, PACKAGE = "Matrix")
+              crit <- max(abs(eta - etaold)) / (0.1 + max(abs(eta)))
+              if (gVerb) cat(sprintf("%03d: %#11g\n", as.integer(iter), crit))
+              ## use this to determine convergence
+              if (crit < controlvals$tolerance) {
+                  conv <- TRUE
+                  break
+              }
+              etaold[] <- eta
+
+              ## Changing number of iterations on second and
+              ## subsequent iterations.
+              if (firstIter)
+              {
+                  controlvals$niterEM <- 2
+                  controlvals$msMaxIter <- 10
+                  firstIter <- FALSE
+              }
+          }
+          if (!conv) warning("IRLS iterations for glmm did not converge")
+          obj
+      })
+
 ## calculates degrees of freedom for fixed effects Wald tests
 ## This is a placeholder.  The answers are generally wrong.  It will
 ## be very tricky to decide what a 'right' answer should be with
@@ -264,7 +424,7 @@ setMethod("logLik", signature(object="lmer"),
               val <- -deviance(object, REML = REML)/2
               nc <- object@nc[-seq(a = object@Omega)]
               attr(val, "nall") <- attr(val, "nobs") <- nc[2]
-              attr(val, "df") <- nc[1] + length(coef(object))
+              attr(val, "df") <- nc[1] + length(ccoef(object))
               attr(val, "REML") <- REML 
               class(val) <- "logLik"
               val
@@ -386,12 +546,7 @@ setMethod("confint", signature(object = "lmer"),
           ci
       })
 
-setReplaceMethod("coef", signature(object = "lmer", value = "numeric"),
-                 function(object, unconst = FALSE, ..., value)
-                 .Call("lmer_coefGets", object, as.double(value),
-                       unconst, PACKAGE = "Matrix"))
-
-setMethod("coef", signature(object = "lmer"),
+setMethod("param", signature(object = "lmer"),
           function(object, unconst = FALSE, ...) {
               .Call("lmer_coef", object, unconst, PACKAGE = "Matrix")
           })
@@ -432,28 +587,6 @@ setMethod("vcov", signature(object = "lmer"),
               rr
           })
 
-Lind <- function(i,j) {
-    if (i < j) stop(paste("Index i=", i,"must be >= index j=", j))
-    ((i - 1) * i)/2 + j
-}
-
-Dhalf <- function(from) {
-    D <- from@D
-    nf <- length(D)
-    Gp <- from@Gp
-    res <- array(0, rep(Gp[nf+1],2))
-    for (i in 1:nf) {
-        DD <- D[[i]]
-        dd <- dim(DD)
-        for (k in 1:dd[3]) {
-            mm <- array(DD[ , , k], dd[1:2])
-            base <- Gp[i] + (k - 1)*dd[1]
-            res[cbind(c(base + row(mm)), c(base + col(mm)))] <- c(mm)
-        }
-    }
-    res
-}
-
 ## Extract the L matrix 
 setAs("lmer", "dtTMatrix",
       function(from)
@@ -511,3 +644,60 @@ setAs("lmer", "dsTMatrix",
       new("dsTMatrix", Dim = as.integer(c(nZ, nZ)), i = Zi, j = Zj, x = Zx,
           uplo = "U")
   })
+
+setMethod("fitted", signature(object = "lmer"),
+          function(object, ...) object@fitted)
+
+setMethod("residuals", signature(object = "lmer"),
+          function(object, ...) object@residuals)
+
+setMethod("coef", signature(object = "lmer"),
+          function(object, ...)
+      {
+          fef <- data.frame(rbind(fixef(object)), check.names = FALSE)
+          ref <- as(ranef(object), "list")
+          names(ref) <- names(object@flist)
+          val <- lapply(ref, function(x) fef[rep(1, nrow(x)),])
+          for (i in seq(a = val)) {
+              refi <- ref[[i]]
+              row.names(val[[i]]) <- row.names(refi)
+              if (!all(names(refi) %in% names(fef)))
+                  stop("unable to align random and fixed effects")
+              val[[i]][ , names(refi)] <- val[[i]][ , names(refi)] + refi
+          }
+          new("lmer.coef", val)
+      })
+
+setMethod("plot", signature(x = "lmer.coef"),
+          function(x, y, ...)
+      {
+          varying <- unique(do.call("c",
+                                    lapply(x, function(el)
+                                           names(el)[sapply(el,
+                                                            function(col)
+                                                            any(col != col[1]))])))
+          gf <- do.call("rbind", lapply(x, "[", j = varying))
+          gf$.grp <- factor(rep(names(x), sapply(x, nrow)))
+          switch(min(length(varying), 3),
+                 qqmath(eval(substitute(~ x | .grp,
+                                        list(x = as.name(varying[1])))), gf, ...),
+                 xyplot(eval(substitute(y ~ x | .grp,
+                                        list(y = as.name(varying[1]),
+                                             x = as.name(varying[2])))), gf, ...),
+                 splom(~ gf | .grp, ...))
+      })
+
+setMethod("plot", signature(x = "lmer.ranef"),
+          function(x, y, ...)
+      {
+          lapply(x, function(x) {
+              cn <- lapply(colnames(x), as.name)
+              switch(min(ncol(x), 3),
+                     qqmath(eval(substitute(~ x, list(x = cn[[1]]))), x, ...),
+                     xyplot(eval(substitute(y ~ x, list(y = cn[[1]], x = cn[[2]]))),
+                            x, ...),
+                     splom(~ x, ...))
+          })
+      })
+
+
